@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>/*4 sleep only*/
+#include <gdk/gdk.h>
 #include "mag_ctrl.h"
 #include "mag.h"
 #include "magxmlapi.h"
@@ -69,25 +70,28 @@ this will be the same for the entire collention of zoomers, so for the entire sc
 schema = a collection of zoomers
 */
 gboolean zoom_factor_changed = TRUE;
+gboolean zoom_factor_was_changed = FALSE; /* does the same thing as zoom_factor_changed but only for the zoom factor*/
 
-gboolean cursor		= TRUE;
-gboolean cursor_scale	= TRUE;
-gboolean crosswire	= TRUE;
+static gboolean cursor		= TRUE;
+static gboolean cursor_scale	= TRUE;
+static gboolean crosswire	= TRUE;
 
-gchar	*cursor_name	= NULL;
-int	cursor_size	= 0;
-guint32	cursor_color	= 0    ;/*color as AlphaRedGreenBlue*/
-float	cursor_zoom_factor = 1;
+static gchar	*cursor_name	= NULL;
+static int	cursor_size	= 0;
+static guint32	cursor_color	= 0    ;/*color as AlphaRedGreenBlue*/
+static float	cursor_zoom_factor = 1;
 
 int      	crosswire_size  = 0    ;/*thickness of crosswire cursor*/
 guint32		crosswire_color = 0    ;/*color as AlphaRedGreenBlue*/
 gboolean	crosswire_clip  = FALSE;/*whenever to insert the cursor over the crosswire or not*/
 
+MagRectangle display_size;
 MagRectangle source_rect;
-MagRectangle target_rect;
 static MagRectangle rect;
+static MagRectangle mouse_tracked_roi;
 
 static MagRectangle roi_viewport;
+extern MagRectangle zp_rect;
 
 /*__DEBUG FUNCION*/
 void 
@@ -192,8 +196,6 @@ mag_zoomer_init (MagZoomer *mag_zoomer)
 
 	mag_zoomer->zoomer_placement.border.width   = BORDER_WIDTH;
 	mag_zoomer->zoomer_placement.border.color   = BORDER_COLOR;
-
-	cursor_zoom_factor = mag_zoomer->params.zoom_factor_x;
 }
 
 void 
@@ -288,6 +290,20 @@ mag_zoomer_set_source (MagZoomer *mag_zoomer,
 			    	     mag_zoomer->source);
 }
 
+void 
+mag_zoomer_set_target (MagZoomer *mag_zoomer, 
+			gchar     *target)
+{
+	if (mag_zoomer->target) 
+	{
+	    g_free (mag_zoomer->target);	
+	    mag_zoomer->target = NULL;
+	}
+	mag_zoomer->target = g_strdup (target);
+
+	magnifier_set_target_screen (magnifier, 
+			    	     mag_zoomer->target);
+}
 
 /* __ MAG ZOOMER PARAMS METHODS ________________________________________________*/
 void 
@@ -305,6 +321,7 @@ mag_zoomer_set_params_zoom_factor (MagZoomer *mag_zoomer,
 	/*check if the zoom factor was changed - 
 	needed for the focus/mouse tracking logic*/
 		zoom_factor_changed = TRUE;
+		zoom_factor_was_changed = TRUE;
 	    if ( (strncasecmp ( "+", zoom_factor_x, 1) != 0) &&
 		 (strncasecmp ( "-", zoom_factor_x, 1) != 0)	     
 	       ) 
@@ -317,6 +334,7 @@ mag_zoomer_set_params_zoom_factor (MagZoomer *mag_zoomer,
 	/*check if the zoom factor was changed - 
 	needed for the focus/mouse tracking logic*/
 	    	zoom_factor_changed = TRUE;
+		zoom_factor_was_changed = TRUE;
 	    if ( (strncasecmp ( "+", zoom_factor_y, 1) != 0) &&
 		 (strncasecmp ( "-", zoom_factor_y, 1) != 0)	     
 	       ) 
@@ -326,7 +344,7 @@ mag_zoomer_set_params_zoom_factor (MagZoomer *mag_zoomer,
 	
 	if (mag_zoomer->params.zoom_factor_x < 1) mag_zoomer->params.zoom_factor_x = 1;
 	if (mag_zoomer->params.zoom_factor_y < 1) mag_zoomer->params.zoom_factor_y = 1;
-	
+
 	magnifier_set_magnification (magnifier, 
 				     0, 
 				     mag_zoomer->params.zoom_factor_x,
@@ -361,19 +379,36 @@ mag_zoomer_set_params_contrast (MagZoomer *mag_zoomer,
 	mag_zoomer->params.contrast = atof (contrast);	
 }
 
+gboolean
+check_for_focus_tracking_none  (MagZoomer *mag_zoomer)
+{
+    if((mag_zoomer->params.alignment.x == MAG_ALIGN_NONE) &&
+       (mag_zoomer->params.alignment.y == MAG_ALIGN_NONE) && zoom_factor_was_changed)
+       return TRUE;
+    else
+	return FALSE;
+}
+
 void 
 mag_zoomer_set_params_tracking (MagZoomer *mag_zoomer, 
 			    	gchar     *tracking)
 {
 	if  (g_strcasecmp(tracking, "focus")  == 0) 
 	{
-	    mag_zoomer->params.tracking = MAG_TRACKING_FOCUS;	
+	    if (check_for_focus_tracking_none (mag_zoomer))
+		mag_zoomer->params.tracking = MAG_TRACKING_MOUSE;	
+	    else
+	    	mag_zoomer->params.tracking = MAG_TRACKING_FOCUS;	
 	}
 	else
 	{
 	    if  (g_strcasecmp(tracking, "mouse")  == 0) 
 	    {
 		mag_zoomer->params.tracking = MAG_TRACKING_MOUSE;
+		mouse_tracked_roi.left = mag_zoomer->region_of_interest.left;
+		mouse_tracked_roi.top = mag_zoomer->region_of_interest.top;
+		mouse_tracked_roi.width = mag_zoomer->region_of_interest.width;
+		mouse_tracked_roi.height = mag_zoomer->region_of_interest.height;
 	    }
 	    else
 	    {
@@ -423,9 +458,70 @@ mag_zoomer_set_params_alignment (MagZoomer *mag_zoomer,
 				 gchar	   *alignment_y)
 {
 	if (alignment_x) 
-	    mag_zoomer->params.alignment.x = atoi (alignment_x);	
-	if (alignment_y)
-	    mag_zoomer->params.alignment.y = atoi (alignment_y);	
+	{
+	    if  (g_strcasecmp(alignment_x, "none")  == 0) 
+	    {
+		mag_zoomer->params.alignment.x = MAG_ALIGN_NONE;	
+	    }
+	    else
+	    {
+		if  (g_strcasecmp(alignment_x, "centered")  == 0) 
+		{
+		    mag_zoomer->params.alignment.x = MAG_ALIGN_CENTER;
+		}
+		else
+		{
+		    if  (g_strcasecmp(alignment_x, "min")  == 0) 
+		    {
+	    	        mag_zoomer->params.alignment.x = MAG_ALIGN_MIN;		
+		    }
+		    else
+		    {
+			if  (g_strcasecmp(alignment_x, "max")  == 0) 
+			{
+	    		    mag_zoomer->params.alignment.x = MAG_ALIGN_MAX;		
+			}
+			else
+			{
+	    		    mag_zoomer->params.alignment.x = MAG_ALIGN_AUTO;		
+			}
+		    }
+		}
+	    }
+	}
+
+	if (alignment_y) 
+	{
+	    if  (g_strcasecmp(alignment_y, "none")  == 0) 
+	    {
+		mag_zoomer->params.alignment.y = MAG_ALIGN_NONE;	
+	    }
+	    else
+	    {
+		if  (g_strcasecmp(alignment_y, "centered")  == 0) 
+		{
+		    mag_zoomer->params.alignment.y = MAG_ALIGN_CENTER;
+		}
+		else
+		{
+		    if  (g_strcasecmp(alignment_y, "min")  == 0) 
+		    {
+	    	        mag_zoomer->params.alignment.y = MAG_ALIGN_MIN;		
+		    }
+		    else
+		    {
+			if  (g_strcasecmp(alignment_y, "max")  == 0) 
+			{
+	    		    mag_zoomer->params.alignment.y = MAG_ALIGN_MAX;		
+			}
+			else
+			{
+	    		    mag_zoomer->params.alignment.y = MAG_ALIGN_AUTO;		
+			}
+		    }
+		}
+	    }
+	}
 }
 
 void 
@@ -469,10 +565,10 @@ mag_zoomer_set_ROI_left (MagZoomer *mag_zoomer,
 {
 	int roi_left = atoi (ROILeft);
 
-	if ( roi_left > source_rect.left)
-	    mag_zoomer->region_of_interest.left = roi_left;
-	else
-	    mag_zoomer->region_of_interest.left = source_rect.left;
+	if (check_for_focus_tracking_none (mag_zoomer))
+	    roi_left = mouse_tracked_roi.left;
+
+        mag_zoomer->region_of_interest.left = roi_left;
 }	
 
 void 
@@ -481,10 +577,10 @@ mag_zoomer_set_ROI_top (MagZoomer *mag_zoomer,
 {
 	int roi_top = atoi (ROITop);
 
-	if ( roi_top > source_rect.top)
-	    mag_zoomer->region_of_interest.top = roi_top;
-	else
-	    mag_zoomer->region_of_interest.top = source_rect.top;	
+	if (check_for_focus_tracking_none (mag_zoomer))
+	    roi_top = mouse_tracked_roi.top;
+        
+	mag_zoomer->region_of_interest.top = roi_top;
 	
 }
 
@@ -493,11 +589,11 @@ mag_zoomer_set_ROI_width (MagZoomer *mag_zoomer,
 			  gchar     *ROIWidth)
 {
 	int roi_width = atoi (ROIWidth);
-
-	if ( roi_width < source_rect.width)
-	    mag_zoomer->region_of_interest.width = roi_width;
-	else
-	    mag_zoomer->region_of_interest.width = source_rect.width; 	
+	
+	if (check_for_focus_tracking_none (mag_zoomer))
+	    roi_width = mouse_tracked_roi.width;
+        
+	mag_zoomer->region_of_interest.width = roi_width;
 }
 
 void 
@@ -505,11 +601,14 @@ mag_zoomer_set_ROI_height (MagZoomer *mag_zoomer,
 			   gchar     *ROIHeight)
 {
 	int roi_height = atoi (ROIHeight);
-
-	if ( roi_height < source_rect.height)
-    	    mag_zoomer->region_of_interest.height = roi_height;
-	else
-    	    mag_zoomer->region_of_interest.height = source_rect.height;	
+	
+	if (check_for_focus_tracking_none (mag_zoomer))
+	{
+	    roi_height = mouse_tracked_roi.height;
+	    zoom_factor_was_changed = FALSE;
+	}
+        
+	mag_zoomer->region_of_interest.height = roi_height;
 }
 
 /* __ MAG ZOOMER ZP METHODS ____________________________________________________*/
@@ -525,16 +624,22 @@ mag_zoomer_set_ZP_extents_left (MagZoomer *mag_zoomer,
 
     zp_left = atoi (ZPLeft);
     
+    zp_rect.left = zp_left;    
+    
     zoom_factor_changed = TRUE;
     
-    if ( zp_left > target_rect.left)
+    if ( zp_left > display_size.left)
 	(mag_zoomer->zoomer_placement).extents.left = zp_left;
     else
-	(mag_zoomer->zoomer_placement).extents.left = target_rect.left;    
+	(mag_zoomer->zoomer_placement).extents.left = display_size.left;    
 
     magnifier_set_target (magnifier,
 			 &((mag_zoomer->zoomer_placement).extents));
-
+    /* get_source has to be called every time set_target is called , 
+    because the source changes*/			 
+    magnifier_get_source (magnifier,
+			      &source_rect);
+			 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
     temp_rect.height = mag_zoomer->zoomer_placement.extents.height - mag_zoomer->zoomer_placement.extents.top;
@@ -555,15 +660,20 @@ void mag_zoomer_set_ZP_extents_top (MagZoomer *mag_zoomer,
 
     zp_top = atoi (ZPTop);
     
+    zp_rect.top = zp_top;
+    
     zoom_factor_changed = TRUE;
 
-    if ( zp_top > target_rect.top)
+    if ( zp_top > display_size.top)
 	mag_zoomer->zoomer_placement.extents.top = zp_top;
     else
-	mag_zoomer->zoomer_placement.extents.top = target_rect.top;    
+	mag_zoomer->zoomer_placement.extents.top = display_size.top;    
 
     magnifier_set_target (magnifier,
 			 &((mag_zoomer->zoomer_placement).extents));
+			 
+    magnifier_get_source (magnifier,
+			      &source_rect);
 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
@@ -586,15 +696,20 @@ mag_zoomer_set_ZP_extents_width (MagZoomer *mag_zoomer,
 	
     zp_width = atoi (ZPWidth);
     
+    zp_rect.width = zp_width;
+
     zoom_factor_changed = TRUE;
 
-    if ( zp_width < target_rect.width)
+    if ( zp_width < display_size.width)
         mag_zoomer->zoomer_placement.extents.width = zp_width;
     else
-        mag_zoomer->zoomer_placement.extents.width = target_rect.width;	
+        mag_zoomer->zoomer_placement.extents.width = display_size.width;	
 
     magnifier_set_target (magnifier,
 			 &((mag_zoomer->zoomer_placement).extents));
+			 
+    magnifier_get_source (magnifier,
+			      &source_rect);
 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
@@ -616,16 +731,21 @@ mag_zoomer_set_ZP_extents_height (MagZoomer *mag_zoomer,
 	return;
 
     zp_height = atoi (ZPHeight);
-
+    
+    zp_rect.height = zp_height;
+    
     zoom_factor_changed = TRUE;
-	
-    if ( zp_height < target_rect.height)
+    	
+    if ( zp_height < display_size.height)
         mag_zoomer->zoomer_placement.extents.height = zp_height;
     else
-        mag_zoomer->zoomer_placement.extents.height = target_rect.height;	    
+        mag_zoomer->zoomer_placement.extents.height = display_size.height;	    
 
     magnifier_set_target (magnifier,
 			 &((mag_zoomer->zoomer_placement).extents));
+    
+    magnifier_get_source (magnifier,
+			      &source_rect);
 
     temp_rect.left = temp_rect.top = 0;
     temp_rect.width = mag_zoomer->zoomer_placement.extents.width - mag_zoomer->zoomer_placement.extents.left;
@@ -656,7 +776,7 @@ mag_zoomers_init ()
 {
 	mag_zoomers_hash_table = g_hash_table_new (g_str_hash, g_str_equal);
 	
-
+	
 }
 
 void 
@@ -697,7 +817,7 @@ mag_zoomers_set_crosswire_size (gchar *size)
 	if (size) 
 	{
 	    crosswire_size = atoi(size);    
-	    if (cursor && crosswire)
+	    if (crosswire)
 		magnifier_set_crosswire_size (magnifier,
 				    	      crosswire_size);
 	}
@@ -710,7 +830,7 @@ mag_zoomers_set_crosswire_color (gchar *color)
     {
 	crosswire_color = (guint32)strtoll(color, &color, 10);
 
-	if (cursor && crosswire)
+	if (crosswire)
 	    magnifier_set_crosswire_color (magnifier,
 					   crosswire_color);
     }
@@ -730,7 +850,7 @@ mag_zoomers_set_crosswire_clip (gchar *clip)
 	{
 	    crosswire_clip = TRUE;
 	}
-	if (cursor && crosswire)
+	if (crosswire)
     	    magnifier_set_crosswire_clip (magnifier,
 				    	  crosswire_clip);
 }
@@ -753,12 +873,10 @@ mag_zoomers_set_cursor (gchar *new_cursor_name,
 	cursor_zoom_factor = atof (new_cursor_zoom_factor);
 	
 #ifdef DEBUG
-	fprintf (stderr, "\n mag : Cursor name %s, cursor size %d, cursor scale %g",
+	fprintf (stderr, "\nmag : Cursor name %s, cursor size %d, cursor_zoom_factor %2g",
 			 cursor_name, cursor_size, cursor_zoom_factor);
 #endif
-
 	if (cursor)
-/*	    if (cursor_name && cursor_size) */
 	    if (cursor_scale)
 	    {
 		    magnifier_set_cursor (magnifier,
@@ -802,20 +920,15 @@ mag_zoomers_set_cursor_color (gchar *color)
 void 
 mag_zoomers_set_cursor_scale_on_off (gchar *cursor_scale_on_off)
 {
-/*	fprintf (stderr, "\n%s %d : cursor mag  %s", 
-			    __FILE__,
-			    __LINE__,
-			    cursor_scale_on_off);
-*/			    
 	if ( (g_strcasecmp(cursor_scale_on_off, "no")    == 0) ||
 	     (g_strcasecmp(cursor_scale_on_off, "false") == 0) ||
 	     (g_strcasecmp(cursor_scale_on_off, "0")     == 0))
 	{
-	    cursor_scale = FALSE;
+	    cursor_scale = TRUE;
 	}
 	else
 	{
-	    cursor_scale = TRUE;
+	    cursor_scale = FALSE;
 	}
 
 	if (cursor_scale)
@@ -860,13 +973,16 @@ mag_zoomers_set_cursor_on_off (gchar *cursor_on_off)
 
 	if (cursor)
 	{
-	    magnifier_set_cursor (magnifier,
-				  cursor_name, 
-				  cursor_size, 
-				  cursor_zoom_factor);
-	    if (crosswire)
-    		magnifier_set_crosswire_size (magnifier,
-					  crosswire_size);
+	    if (cursor_scale)
+    		magnifier_set_cursor (magnifier,
+				    cursor_name, 
+				    CURSOR_SIZE_INVALID, 
+				    cursor_zoom_factor);
+	    else
+		magnifier_set_cursor (magnifier,
+				    cursor_name, 
+				    cursor_size, 
+				    cursor_zoom_factor);
 	}
 	else
 	{
@@ -874,8 +990,6 @@ mag_zoomers_set_cursor_on_off (gchar *cursor_on_off)
 				  "none", 
 				  0, 
 				  1);
-	    magnifier_set_crosswire_size (magnifier,
-					  0);
 	}
 
 }
@@ -1029,7 +1143,6 @@ void mag_tracking_logic (MagZoomer *rv, int out)
     int align_x, align_y;
     int width, height;
 
-
  	width  = (rv->zoomer_placement.extents.width-
 		  rv->zoomer_placement.extents.left) / (rv->params.zoom_factor_x);
 	height = (rv->zoomer_placement.extents.height-
@@ -1060,9 +1173,12 @@ void mag_tracking_logic (MagZoomer *rv, int out)
 	{
 	    case MAG_TRACKING_FOCUS:
 		
-		if (out) 
+		if (out || 
+		    rv->params.alignment.x !=  MAG_ALIGN_AUTO ||
+		    rv->params.alignment.y !=  MAG_ALIGN_AUTO) 
 		{
-		    if (out & OUT_X)
+		    if ( (out & OUT_X) ||
+			 rv->params.alignment.x !=  MAG_ALIGN_AUTO) 
 		    {
 			align_x = rv->params.alignment.x;
 			if (align_x == MAG_ALIGN_AUTO)
@@ -1092,11 +1208,15 @@ void mag_tracking_logic (MagZoomer *rv, int out)
 	    		    case MAG_ALIGN_MAX:
 				roi_point.x = rv->region_of_interest.width - width;
 				break;
+			    default:
+			    	roi_point.x = roi_viewport.left;
+				break;
 			}
 			roi_viewport.left = roi_point.x ;
-			roi_viewport.width   = roi_viewport.left + width;
+			roi_viewport.width = roi_viewport.left + width;
 		    }    
-		    if (out & OUT_Y)
+		    if ( (out & OUT_Y) ||
+			rv->params.alignment.y !=  MAG_ALIGN_AUTO) 
 		    {
 			align_y = rv->params.alignment.y;
 			
@@ -1121,6 +1241,7 @@ void mag_tracking_logic (MagZoomer *rv, int out)
 				roi_point.y = rv->region_of_interest.height - height;	    
 				break;
 			    default:
+			    	roi_point.y = roi_viewport.top;	    
 				break;
 			}
 			roi_viewport.top  = roi_point.y ;	
@@ -1213,14 +1334,13 @@ mag_set_properties (MagZoomer *rv)
 	if (zoom_factor_changed)
 	{
 	    out = OUT_LEFT | OUT_RIGHT | OUT_TOP | OUT_BOTTOM;
-    	    cursor_zoom_factor = (rv->params.zoom_factor_x < rv->params.zoom_factor_y)?
-				  rv->params.zoom_factor_x:
-				  rv->params.zoom_factor_y;
+    	    cursor_zoom_factor = (rv->params.zoom_factor_x + rv->params.zoom_factor_y)/2;
 #ifdef MAG_DEBUG
-	    fprintf (stderr,"%s %d : zoom_factor_changed %d",
+	    fprintf (stderr,"\n%s %d (mag_set_properties): zoom_factor_changed %d, cursor_zoom_factor %2g",
 			    __FILE__,
 			    __LINE__,
-			    zoom_factor_changed);
+			    zoom_factor_changed,
+			    cursor_zoom_factor);
 #endif
 	    if (cursor_scale)
 	    {
@@ -1306,9 +1426,8 @@ mag_add_zoomer (MagZoomer *mag_zoomer)
 		{
 #ifdef MAG_DEBUG
 			fprintf (stderr, "\nmag : Zoomer found : ");
-			mag_zoomer_content(rv);
+			mag_zoomer_content (rv);
 #endif
-
 		}
 		else
 		{
@@ -1351,6 +1470,7 @@ int
 mag_initialize (MagEventCB mag_event)
 {
 	MagRectangle roi;
+	MagRectangle target_rect;
 
 	mag_zoomers_init ();
 	magnifier = get_magnifier ();
@@ -1380,15 +1500,19 @@ mag_initialize (MagEventCB mag_event)
 			 target_rect.width,
 			 target_rect.height);
 #endif
+	display_size.left = display_size.top = DEFAULT_SCREEN_MIN_SIZE;
+	display_size.width = gdk_screen_width () - 1;
+	display_size.height = gdk_screen_height () - 1;
+	
         if (!srconf_set_data (MAGNIFIER_DISPLAY_SIZE_X, 
 			      CFGT_INT, 
-			      (gpointer)&target_rect.width, 
+			      (gpointer)&display_size.width, 
 			      MAGNIFIER_CONFIG_PATH)
 	) return FALSE;
 
         if (!srconf_set_data (MAGNIFIER_DISPLAY_SIZE_Y, 
 			      CFGT_INT, 
-			      (gpointer)&target_rect.height, 
+			      (gpointer)&display_size.height, 
 			      MAGNIFIER_CONFIG_PATH)
 	) return FALSE;
 
@@ -1410,7 +1534,7 @@ mag_initialize (MagEventCB mag_event)
 	magnifier_create_region (magnifier, 2.0, 2.0, &roi , &roi);			 
 
 	magnifier_get_viewport (magnifier, 0, &rect);
-
+	
 	return TRUE;
 }
 
@@ -1425,5 +1549,3 @@ mag_terminate ()
 	fprintf(stderr,"\nTerminate the magnifier session\n");
 #endif
 }
-
-
