@@ -44,9 +44,6 @@ extern gboolean src_use_speech;
 extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
 extern gboolean src_enable_format_string;
-extern gchar   *src_braille_translation_table;
-extern gint	src_braille_position_sensor;
-extern gint	src_braille_optical_sensor;
 
 gboolean  src_button_left_pressed   = FALSE;
 gboolean  src_button_middle_pressed = FALSE;
@@ -880,7 +877,6 @@ src_nav_logic (SRNavigationDir dir, const char *missing_node)
 static gboolean src_flat_review_next_line ();
 static gboolean src_flat_review_prev_line ();
 
-extern gint src_braille_offset;
 static gint crt_line = 0;
 
 static gboolean
@@ -888,7 +884,7 @@ src_nav_parent_flat ()
 {
     gboolean rv = FALSE;
     
-    src_braille_offset = 0;
+    src_braille_set_offset (0);
     rv = src_flat_review_prev_line();
 
     return rv;
@@ -913,7 +909,7 @@ src_nav_child_flat ()
 {
     gboolean rv = FALSE;
     
-    src_braille_offset = 0;
+    src_braille_set_offset (0);
     rv = src_flat_review_next_line();
     
     return rv;
@@ -939,8 +935,12 @@ src_nav_next_flat ()
 {
     gboolean rv = FALSE;
     gboolean old_value = src_use_speech;
+    gint offset;
+    
+    offset = src_braille_get_offset ();
+    offset ++;
+    src_braille_set_offset (offset);
     
-    src_braille_offset++;
     src_use_speech = FALSE;
     src_flat_review_line (crt_line);
     src_use_speech = old_value;
@@ -1159,8 +1159,10 @@ src_nav_previous_flat ()
 {
     gboolean rv = FALSE;
     gboolean old_value = src_use_speech;
+    gint offset;
     
-    src_braille_offset = MAX (0, src_braille_offset - 1);
+    offset = src_braille_get_offset ();
+    src_braille_set_offset (MAX (0, offset - 1));
     
     src_use_speech = FALSE;
     src_flat_review_line (crt_line);
@@ -2424,10 +2426,16 @@ src_present_line_to_braille_or_brlmon (SRWAccLine *line)
     if (brlout)
     {
 	gchar *tmp;
+	gchar *braille_translation_table = NULL;
 	gchar offset[10];
+	gint braille_offset;
+	
 	tmp = brlout;
-	sprintf (offset, "%d", src_braille_offset);
-	brlout = g_strconcat ("<BRLOUT language=\"", src_braille_translation_table, "\" bstyle=\"8\" clear=\"true\">",
+	braille_translation_table = src_braille_get_translation_table ();
+	braille_offset = src_braille_get_offset ();
+	
+	sprintf (offset, "%d", braille_offset);
+	brlout = g_strconcat ("<BRLOUT language=\"", braille_translation_table, "\" bstyle=\"8\" clear=\"true\">",
 				"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">",
 				brlout, "</BRLDISP>",
 				"</BRLOUT>",
@@ -2818,7 +2826,7 @@ src_ctrl_flat_mode_init ()
 {
     gboolean rv = FALSE;
     
-    src_braille_offset = 0;
+    src_braille_set_offset (0);
     src_tracking_mode = SRC_MODE_FLAT_REVIEW;
     if (src_use_speech)
 	src_say_message (_("flat review mode")); 
@@ -2919,7 +2927,10 @@ static gboolean
 src_braille_scroll_char_left ()
 {
     gboolean old = src_use_speech;
-    src_braille_offset = MAX (0, src_braille_offset - 1);
+    gint offset;
+    
+    offset = src_braille_get_offset ();
+    src_braille_set_offset (MAX (0, offset - 1));
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -2931,7 +2942,10 @@ static gboolean
 src_braille_scroll_char_right ()
 {
     gboolean old = src_use_speech;
-    src_braille_offset = MIN (1024, src_braille_offset + 1);
+    gint offset;
+    
+    offset = src_braille_get_offset ();
+    src_braille_set_offset (MIN (1024, offset + 1));
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -2945,7 +2959,10 @@ static gboolean
 src_braille_scroll_display_left ()
 {
     gboolean old = src_use_speech;
-    src_braille_offset = MAX (0, src_braille_offset - 20);
+    gint offset;
+    
+    offset = src_braille_get_offset ();
+    src_braille_set_offset (MAX (0, offset - 20));
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -2959,7 +2976,10 @@ static gboolean
 src_braille_scroll_display_right ()
 {
     gboolean old = src_use_speech;
-    src_braille_offset = MIN (1024, src_braille_offset + 20);
+    gint offset;
+    
+    offset = src_braille_get_offset ();
+    src_braille_set_offset (MIN (1024, offset + 20));
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -3545,12 +3565,15 @@ src_ctrl_position_sensor_action_flat (gint index)
 static void
 src_ctrl_position_sensor_action_logic (gint index)
 {
+    gint braille_position_sensor;
+    
     if (!src_crt_sro)
 	return;
 	
     src_text_index = index >= 0 ? index : 0;
 
-    switch (src_braille_position_sensor)
+    braille_position_sensor = src_braille_get_position_sensor ();
+    switch (braille_position_sensor)
     {
 	case 0:
 	    break;
@@ -3619,12 +3642,14 @@ src_ctrl_position_sensor_action_logic (gint index)
 static void
 src_ctrl_optical_sensor_action_logic (gint index)
 {
+    gint braille_optical_sensor;
     if (!src_crt_sro)
 	return;
 
     src_text_index = index >= 0 ? index : 0;
 
-    switch (src_braille_optical_sensor)
+    braille_optical_sensor = src_braille_get_optical_sensor ();
+    switch (braille_optical_sensor)
     {
 	case 0:
 	    break;
